# -*- encoding: utf-8 -*-
#   Copyright 2011 Agilo Software GmbH All rights reserved
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import urllib2, base64

from agilo.test.functional import AgiloFunctionalTestCase
from agilo.test import Usernames
from agilo.utils.config import AgiloConfig

class TestXMLRPCFetching(AgiloFunctionalTestCase):
    
    def setUp(self):
        self.super()
        self.tester.login_as(Usernames.admin)
        

        components = AgiloConfig(self.env).get_section('components')
        components.change_option("tracrpc.*", 'enabled')
        components.save()
        
        self.ticket1_id = self.tester.create_new_agilo_task('foo', 'abc')
        self.ticket2_id = self.tester.create_new_agilo_task('bar', 'def')
        
    def _should_skip_test(self):
        try:
            import tracrpc
        except:
            return True
        return False
    
    def _send_rpc_request(self, body):
        request = urllib2.Request(self.tester.url + self.env.base_url + '/login/rpc', data=body)
        request.add_header('Content-Type', 'application/xml')
        request.add_header('Content-Length', str(len(body)))
        request.add_header('Authorization', 'Basic %s' \
                                 % base64.encodestring('admin:admin')[:-1])
        self.assertEquals('POST', request.get_method())
        response = urllib2.urlopen(request)
        self.assertEquals(200, response.code)
        return response

    def _request_body_for_fetch(self, ticket_id):
        return """<?xml version="1.0"?>
                    <methodCall>
                        <methodName>ticket.get</methodName>
                        <params>
                            <param><int>%s</int></param>
                        </params>
                    </methodCall>""" % ticket_id

    def test_can_fetch_a_ticket(self):
        if self._should_skip_test():
            return
        body = self._request_body_for_fetch(self.ticket1_id)
        response = self._send_rpc_request(body)
        response_body = response.read()
        self.assertRegexpMatches(response_body, "<int>%s</int>" % self.ticket1_id)
        self.assertRegexpMatches(response_body, "<value><string>foo</string></value>")
        self.assertRegexpMatches(response_body, "<value><string>abc</string></value>")
        
    def test_can_fetch_ticket_list(self):
        if self._should_skip_test():
            return
        body = """<?xml version="1.0"?>
                    <methodCall>
                        <methodName>ticket.query</methodName>
                            <params>
                                <param><string>max=0</string></param>
                            </params>
                    </methodCall>""" 
        response = self._send_rpc_request(body)
        response_body = response.read()
        self.assertRegexpMatches(response_body, "<int>%s</int>" % self.ticket1_id)
        self.assertRegexpMatches(response_body, "<int>%s</int>" % self.ticket2_id)
        
    def test_cannot_write_tickets(self):
        if self._should_skip_test():
            return
        def _prepare_request_body(method):
            return """<?xml version="1.0"?>
                        <methodCall>
                            <methodName>ticket.%s</methodName>
                            <params>
                                <param><string>test summary</string></param>
                                <param><string>test desc</string></param>
                            </params>
                        </methodCall>""" % method
            
        for method in ['create', 'update']:
            response = self._send_rpc_request(_prepare_request_body(method))
            self.assertRegexpMatches(response.read(), "RPC method \"ticket.%s\" not found" % method)
            
    def test_can_delete_tickets(self):
        if self._should_skip_test():
            return
        body = """<?xml version="1.0"?>
            <methodCall>
                <methodName>ticket.delete</methodName>
                <params>
                    <param><int>%s</int></param>
                </params>
            </methodCall>""" % self.ticket1_id
        self._send_rpc_request(body)

        response = self._send_rpc_request(self._request_body_for_fetch(self.ticket1_id))
        response_body = response.read()
        self.assertRegexpMatches(response_body, "Ticket %s does not exist" % self.ticket1_id )


        